@@ -1,4 +1,4 @@
-#' Create a simulator
+#' Create a Simulator
 #'
 #' This method initialises a simulation environment.
 #'
@@ -38,7 +38,7 @@
 #'
 simmer <- function(name="anonymous", verbose=FALSE) Simmer$new(name, verbose)
 
-#' Reset a simulator
+#' Reset a Simulator
 #'
 #' Reset the following components of a simulation environment:
 #' time, event queue, resources, generators and statistics.
@@ -53,20 +53,34 @@ reset <- function(.env) UseMethod("reset")
 #' @export
 reset.simmer <- function(.env) .env$reset()
 
-#' Run the simulation
+#' Run a Simulation
 #'
-#' Execute steps until the given criterion.
+#' Execute steps until a given criterion.
 #'
 #' @inheritParams reset
 #' @param until stop time.
+#' @param progress optional callback to show the progress of the simulation. The
+#' completed ratio is periodically passed as argument to the callback.
+#' @param steps number of steps to show as progress (it takes effect only if
+#' \code{progress} is provided).
 #'
 #' @return Returns the simulation environment.
 #' @seealso \code{\link{reset}}.
 #' @export
-run <- function(.env, until=1000) UseMethod("run")
+run <- function(.env, until=1000, progress=NULL, steps=10) UseMethod("run")
 
 #' @export
-run.simmer <- function(.env, until=1000) .env$run(until)
+run.simmer <- function(.env, until=1000, progress=NULL, steps=10) {
+  check_args(until, progress, steps, types=c("number", "function or NULL", "number"))
+  if (is.function(progress)) {
+    progress(0)
+    for (i in seq(until/steps, until, until/steps)) {
+      .env$run(until=i)
+      progress(i/until)
+    }
+    .env
+  } else .env$run(until=until)
+}
 
 #' @rdname run
 #' @export
@@ -75,7 +89,7 @@ onestep <- function(.env) UseMethod("onestep")
 #' @export
 onestep.simmer <- function(.env) .env$step()
 
-#' Get current time
+#' Simulation Time
 #'
 #' Get the current simulation time.
 #'
@@ -89,7 +103,7 @@ now <- function(.env) UseMethod("now")
 #' @export
 now.simmer <- function(.env) .env$now()
 
-#' Peek next events
+#' Peek Next Events
 #'
 #' Look for future events in the event queue and (optionally) obtain info about them.
 #'
@@ -105,7 +119,7 @@ peek <- function(.env, steps=1, verbose=FALSE) UseMethod("peek")
 #' @export
 peek.simmer <- function(.env, steps=1, verbose=FALSE) .env$peek(steps, verbose)
 
-#' Add a resource
+#' Add a Resource
 #'
 #' Define a new resource in a simulation environment.
 #'
@@ -139,7 +153,7 @@ add_resource.simmer <- function(.env, name, capacity=1, queue_size=Inf, mon=TRUE
                                 preempt_order=c("fifo", "lifo"), queue_size_strict=FALSE)
   .env$add_resource(name, capacity, queue_size, mon, preemptive, preempt_order, queue_size_strict)
 
-#' Add a generator
+#' Add a Generator
 #'
 #' Define a new generator of arrivals in a simulation environment.
 #'
@@ -174,9 +188,9 @@ add_generator.simmer <- function(.env, name_prefix, trajectory, distribution, mo
                                  priority=0, preemptible=priority, restart=FALSE)
   .env$add_generator(name_prefix, trajectory, distribution, mon, priority, preemptible, restart)
 
-#' Get statistics
+#' Monitoring Statistics
 #'
-#' Simulator getters for obtaining monitored data (if any) about arrivals, attributes and resources.
+#' Getters for obtaining monitored data (if any) about arrivals, attributes and resources.
 #'
 #' @param .envs the simulation environment (or a list of environments).
 #' @param per_resource if \code{TRUE}, statistics will be reported on a per-resource basis.
@@ -210,28 +224,70 @@ get_mon_resources <- function(.envs, data=c("counts", "limits"))
 get_mon_resources.simmer <- function(.envs, data=c("counts", "limits"))
   envs_apply(.envs, "get_mon_resources", data)
 
-#' Get the number of arrivals generated
+#' Get Process Parameters
 #'
-#' Simulator getter for obtaining the number of arrivals generated by a generator by name.
+#' Getters for processes (generators and arrivals) number of arrivals generated
+#' by a generator, the name of the active arrival, an attribute from the active
+#' arrival or a global one, and prioritization values.
 #'
 #' @inheritParams reset
 #' @param generator the name of the generator.
 #'
-#' @return Returns a numeric value.
+#' @details \code{get_n_generated} returns the number of arrivals generated by a
+#' given generator.
+#'
+#' \code{get_name} returns the number of the running arrival. \code{get_attribute}
+#' returns a running arrival's attribute or a global one. If the provided key was
+#' not previously set, it returns a missing value. \code{get_global} is a shortcut
+#' for \code{get_attribute(global=TRUE)}. \code{get_prioritization} returns a
+#' running arrival's prioritization values. \code{get_name}, \code{get_attribute}
+#' and \code{get_prioritization} are meant to be used inside a trajectory; otherwise,
+#' there will be no arrival running and these functions will throw an error.
+#'
+#' @seealso \code{\link{set_attribute}}, \code{\link{set_prioritization}}.
 #' @export
 get_n_generated <- function(.env, generator) UseMethod("get_n_generated")
 
 #' @export
 get_n_generated.simmer <- function(.env, generator) .env$get_n_generated(generator)
 
-#' Get a resource's parameter
+#' @rdname get_n_generated
+#' @export
+get_name <- function(.env) UseMethod("get_name")
+
+#' @export
+get_name.simmer <- function(.env) .env$get_name()
+
+#' @param key the attribute name.
+#' @param global if \code{TRUE}, the attribute will be global instead of per-arrival.
 #'
-#' Getters for a resource's server capacity/count and queue size/count.
+#' @rdname get_n_generated
+#' @export
+get_attribute <- function(.env, key, global=FALSE) UseMethod("get_attribute")
+
+#' @export
+get_attribute.simmer <- function(.env, key, global=FALSE) .env$get_attribute(key, global)
+
+#' @rdname get_n_generated
+#' @export
+get_global <- function(.env, key) get_attribute(.env, key, TRUE)
+
+#' @rdname get_n_generated
+#' @export
+get_prioritization <- function(.env) UseMethod("get_prioritization")
+
+#' @export
+get_prioritization.simmer <- function(.env) .env$get_prioritization()
+
+#' Get Resource Parameters
+#'
+#' Getters for resources: server capacity/count and queue size/count.
 #'
 #' @inheritParams reset
 #' @param resource the name of the resource.
 #'
-#' @return Returns a numeric value.
+#' @return Return a numeric value.
+#' @seealso \code{\link{set_capacity}}, \code{\link{set_queue_size}}.
 #' @export
 get_capacity <- function(.env, resource) UseMethod("get_capacity")
 
