@@ -1,22 +1,22 @@
 context("branch")
 
 test_that("a non-function option fails", {
-  expect_error(create_trajectory() %>% branch(1, T, create_trajectory()))
+  expect_error(trajectory() %>% branch(1, T, trajectory()))
 })
 
 test_that("the wrong number of elements fails", {
-  expect_error(create_trajectory() %>% branch(function() 1, T, create_trajectory(), create_trajectory()))
-  expect_error(create_trajectory() %>% branch(function() 1, c(T, T), create_trajectory()))
+  expect_error(trajectory() %>% branch(function() 1, T, trajectory(), trajectory()))
+  expect_error(trajectory() %>% branch(function() 1, c(T, T), trajectory()))
 })
 
 test_that("an empty trajectory fails", {
-  expect_error(create_trajectory() %>% branch(function() 1, T, create_trajectory()))
+  expect_error(trajectory() %>% branch(function() 1, T, trajectory()))
 })
 
 test_that("an index equal to 0 skips the branch", {
-  t0 <- create_trajectory() %>%
+  t0 <- trajectory() %>%
     branch(function() 0, T,
-           create_trajectory() %>% timeout(1)
+           trajectory() %>% timeout(1)
     ) %>%
     timeout(2)
 
@@ -27,13 +27,13 @@ test_that("an index equal to 0 skips the branch", {
 })
 
 test_that("an index out of range fails", {
-  t1 <- create_trajectory() %>%
+  t1 <- trajectory() %>%
     branch(function() 1, T,
-      create_trajectory() %>% timeout(1)
+      trajectory() %>% timeout(1)
     )
-  t2 <- create_trajectory() %>%
+  t2 <- trajectory() %>%
     branch(function() 2, T,
-      create_trajectory() %>% timeout(1)
+      trajectory() %>% timeout(1)
     )
 
   env <- simmer(verbose = TRUE) %>%
